package article_test

import (
	"fmt"
	"testing"
	"unicode/utf8"
)

func TestStringRange(t *testing.T) {
	var ss = []string{
		"\x00\x01\x02\x03",
		"Hello, World",
		"技術評論社",
		"Pokémon",
	}

	// 紙面幅の都合で紹介していませんが、stringをrangeの引数として使うと
	// range []rune(ss)
	// と同等になり、ssをruneごとにバラしたものをループで使うことができます。
	for _, s := range ss {
		t.Run(fmt.Sprintf("%+q", s), func(t *testing.T) {
			for i, r := range s {
				if utf8.RuneLen(r) > 1 {
					fmt.Printf("i=%2d, r=%q (%+q, %x)\n", i, r, r, string([]rune{r}))
				} else {
					fmt.Printf("i=%2d, r=%q (%x)\n", i, r, string([]rune{r}))
				}
			}
		})
	}
}

func ExampleCodePointTable() {
	s := "技術評論社"

	fmt.Printf("| 字 | code point | UTF-8 符号      |\n")
	fmt.Printf("|----|------------|-----------------|\n")
	for len(s) > 0 {
		r, n := utf8.DecodeRuneInString(s)
		if r == utf8.RuneError {
			return
		}

		fmt.Printf("| %[1]c | U+%-8[1]x | %#-15[2]x |\n", r, s[:n])
		s = s[n:]
	}
	// OUTPUT:
	//| 字 | code point | UTF-8 符号      |
	//|----|------------|-----------------|
	//| 技 | U+6280     | 0xe68a80        |
	//| 術 | U+8853     | 0xe8a193        |
	//| 評 | U+8a55     | 0xe8a995        |
	//| 論 | U+8ad6     | 0xe8ab96        |
	//| 社 | U+793e     | 0xe7a4be        |
}
